(*                                                                          *)
(* (c) 2005 Anastasia Gornostaeva. <ermine@ermine.pp.ru>                    *)
(*                                                                          *)
(* JEP 0077                                                                 *)
(* jabber:iq:register                                                       *)

exception JEP77Error of string

let send_req id type_ subels out =
   out (Xmlelement 
	   ("iq", ["type", type_; "id", id];
	    [Xmlelement ("query", ["xmlns", "jabber:iq:register"], subels)]
	   ))

let analyze_rsp xml id =
   match xml with
      | Xmlelement ("iq", attrs, subels) ->
	   let id1 = try List.assoc "id" attrs with Not_found -> "" in
	   let t = try List.assoc "type" attrs with Not_found -> "" in
	   let xmlns = try get_attr_s xml ~path:["query"] "xmlns" with
		 Not_found -> "" in
	      if id =  id1 && xmlns = "jabber:iq:register" then
		 t, subels
	      else
		 raise (JEP77Error "Unexpected data, cannot work")
      | _ ->
	   raise (JEP77Error "Unexpected data, cannot work")

let create_user username password next_xml out =
   send_req "reg1" "get" [] out;
   let type_, subels = analyze_rsp (next_xml ()) "reg1" in
      match type_ with
	 | "result" ->
	      if List.find (fun s ->
			       match s with
				  | (Xmlelement "instructions". _, _) -> true
				  | _ -> false
			   ) <> [] then
		 (* instructions *)
		 send_req "reg2" "set" [make_simple_cdata "username"
					   username;
					make_simple_cdata "password"
					   password
	      else
		 if List.find (fun s ->
				  match s with
				     | (Xmlelement "registered". _, _) -> true
				  | _ -> false
			   ) <> [] then
		    (* registered *)
		    
...
